#!/usr/bin/python3

import re
import os,json
import logging
import subprocess
import apt_pkg 
from SystemUpdater.Core.utils import get_config_patch
from gettext import gettext as _
from SystemUpdater.Core.errors import *
from SystemUpdater.Core.enums import *

def _scan_essential_pkgs():
    config_path = "/usr/share/kylin-update-desktop-config/config"
    json_files = []
    essential_pkgs = []
    remove_white = []

    # 扫描 config目录下 以.json结尾的配置文件
    for root, dirs, files in os.walk(config_path):
        for file in files:
            if file.endswith('.json'):
                json_files.append(os.path.join(root, file))

    for ifile in json_files:
        try:
            with open(ifile,'r') as f:
                data = json.load(f)
        except Exception as exc:
            logging.error(exc)
            continue
        install_list = data.get("install_list",[])
        remove_white_list = data.get("remove_white_list",[])

        # 去重作合集
        essential_pkgs = list(set(install_list+essential_pkgs))
        remove_white = list(set(remove_white_list+remove_white))

    return essential_pkgs,remove_white

class UpdateEssentialItem:
    def __init__(self):
        self.essential_pkgs = []
        self.remove_white = []

        self.read_path = get_config_patch() + 'kylin-update-desktop-system.json'
        self.update()

    def update(self):
        if os.path.exists(self.read_path):
            with open(self.read_path,'r') as f:
                try:
                    data = json.load(f)
                    self.essential_pkgs = data['install_list']
                except Exception as exc:
                    logging.warning(exc)
                try:
                    self.remove_white = data['remove_white_list']
                except Exception as exc:
                    pass
                    # logging.warning(exc)

    def remove_essential(self,remove_pkgs):
        logging.info("Check: For remove of Essential Packages...")
        for pkg in remove_pkgs:
            if pkg in self.essential_pkgs:
                logging.error("Essential Packages(%s) will be Removed...",pkg)
                raise UpdateBaseError(ERROR_REMOVE_ESSENTIAL_PACKAGES)

    def check_white(self,remove_pkgs):
        logging.info("Check: remove pkg in White Packages...")
        new_upgrade_list = list(set(remove_pkgs) & set(self.remove_white))

        for pkg in new_upgrade_list:
            if pkg in remove_pkgs:
                logging.info("%s will be remove in remove pkg...",pkg)
                remove_pkgs.remove(pkg)

class UpdateSanityCheck:
    def __init__(self,cache=None,rootdir=None):
        self.database_path = "/usr/share/kylin-update-desktop-config/version/packages-database.json"
        self.standard_system = "/usr/share/kylin-update-desktop-config/version/standard-system.json"
        self.standard_sources =  "/usr/share/kylin-update-desktop-config/version/standard-sources.json"
        self.rootdir = rootdir
        self.cache = cache
        
        if rootdir:
            rootdir = os.path.abspath(rootdir)
            self.database_path = rootdir + self.database_path
            self.standard_system = rootdir + self.standard_system
            self.standard_sources = rootdir + self.standard_sources

    def check(self):
        sanity_sources = {}
        sanity_system = {}
        packages_database = {}
        desc = ''

        if not os.path.exists(self.database_path) or \
                not os.path.exists(self.standard_system) or not os.path.exists(self.standard_sources) :
            logging.warning("Skip: the pre-update version match check...")
            return True,desc

        try:
            with open(self.database_path,'r') as f:
                packages_database = json.load(f)
        except Exception as exc:
            logging.warning(str(exc))

        try:
            with open(self.standard_system,'r') as f:
                sanity_system = json.load(f)
        except Exception as exc:
            logging.warning(str(exc))

        try:
            with open(self.standard_sources,'r') as f:
                sanity_sources = json.load(f)
        except Exception as exc:
            logging.warning(str(exc))

        # 停止检测功能
        if sanity_system.get("stop_version_match",False):
            return True,desc

        if packages_database and sanity_system and sanity_sources:
            if self.rootdir:
                local_packages = self._get_installed_from_cache(self.cache)
            else:
                local_packages = self._get_installed_from_dpkg()
            
            if not local_packages:
                return True,desc
            else:
                essential_pkgs,_ = _scan_essential_pkgs()
                filter_packages = {pkg: local_packages[pkg] for pkg in essential_pkgs if pkg in local_packages}

                return self._check_sanity_verison(filter_packages,packages_database,sanity_system,sanity_sources)

        return True,desc

    def _check_sanity_verison(self,local_packages,packages_database,sanity_system,sanity_sources):
        try:
            standard_sources = self.is_standard_sources(sanity_sources)
            # 检查定制版本
            customized,customized_popoup = self.is_customized_system(local_packages,sanity_system)
        except Exception as e:
            logging.error(e)
            return True,''

        # 不能升级 定制版本 + 通用源
        if customized and standard_sources:
            # system = 
            # sources = 
            desc = (_("the %s uses a %s.") % (_(CUSTOMIZED_SYSTEM),_(STANDARD_SOURCES)))
            logging.info("Version Check Desc:%s",desc)
            if customized_popoup:
                return False,desc
            else:
                raise UpdateBaseError(ERROR_SYSTEM_SANITY_FAILED,desc=desc)
            
        intermediate,inter_popoup = self.is_intermediate_version(local_packages,sanity_system,packages_database)

        # 中间版本 + 通用源
        if intermediate and standard_sources:
            desc = (_("the %s uses a %s.") % (_(INTERMEDIATE_SYSTEM),_(STANDARD_SOURCES)))
            logging.info("Version Check Desc:%s",desc)
            if inter_popoup:
                return False,desc
            else:
                raise UpdateBaseError(ERROR_SYSTEM_SANITY_FAILED,desc=desc)

        # 不能升级 通过版本 + 定制源 
        if not intermediate and not customized and not standard_sources:
            desc = (_("the %s uses a %s.") % (_(STANDARD_SYSTEM),_(CUSTOMIZED_SOURCES)))
            logging.info("Version Check Desc:%s",desc)
            if customized_popoup:
                return False,desc
            else:
                raise UpdateBaseError(ERROR_SYSTEM_SANITY_FAILED,desc=desc)
        
        return True,''

    def _extract_sources_local(self,file_path):
        deb_lines = []
        if file_path == "sources.list":
            file_path = "/etc/apt/sources.list"

        try:
            with open(file_path, 'r') as file:
                for line in file:
                    if line.startswith('deb http'):
                        deb_lines.append(line.strip())
                    elif line.startswith('deb file'):
                        pass
        except Exception as e:
            logging.error(str(e))
            return []
        return deb_lines

    def is_standard_sources(self,sanity_sources):
        TEST_SOURCE_FLAGE = ".tags--"
        TEST_SOURCE_FLAGE2 = "archive.launchpad.dev/"
        P2P_FLAGS = "localhost:9977/"
        local_sources = []

        sources_path = None
        DIR_MRDIA = "/media/OfflineSource/"
        if os.path.exists(DIR_MRDIA):
            for second_dir in os.listdir(DIR_MRDIA):
                check_dir_two = DIR_MRDIA + second_dir + "/"
                if not os.path.isdir(check_dir_two):
                    continue
                check_file = check_dir_two + "sources.list"
                if os.path.exists(check_file):
                    sources_path = check_file
        if not sources_path:
            sources_path = apt_pkg.config["Dir::Etc::SourceList"]
            if sources_path == "sources.list":
                sources_path = "/etc/apt/sources.list"

        try:
            with open(sources_path, 'r') as file:
                for line in file:
                    if line.startswith('deb http'):
                        # 去除源后面的注释
                        if "#" in line:
                            line = line.split("#")[0]
                        line = line.strip()
                        local_sources.append(line)
                    elif line.startswith('deb file'):
                        pass
        except Exception as e:
            logging.error(str(e))

        if not local_sources:
            return True
        
        all_sources = [value.strip() for values_list in sanity_sources.values() for value in values_list]

        # 不区分https和http 
        # deb http://archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
        # archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
        new_sources = []
        for value in all_sources:
            if "://" in value:
                value = value.split("://")[1]
            new_sources.append(value)

        origin_sources = local_sources
        local_sources = [value.split("://")[1] for value in local_sources]

        # localhost:9977/archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
        if "localhost:" in local_sources:
            # archive.kylinos.cn/kylin/KYLIN-ALL 10.1 main restricted universe multiverse
            local_sources = local_sources.replace(P2P_FLAGS,"")

        # 去除"archive.launchpad.dev/"内源地址
        local_sources = [value.replace(TEST_SOURCE_FLAGE2, "") for value in local_sources]
        # 去除.tags-- 标识
        local_sources = [value.replace(TEST_SOURCE_FLAGE, "") for value in local_sources]
        #去除-- 后面的字符串
        local_sources = [re.sub(r'--.*?\s', ' ', value) for value in local_sources]

        # 警告去除特殊符号后的本地源
        except_sources = []
        for source in local_sources:
            if source not in new_sources:
                except_sources.append(source)

        if except_sources:
            logging.info("Pushed to a Customized patch...")
            logging.info("Customized patch: sources(%r)...",except_sources)
            return False
        else:
            logging.info("Pushed to a standard patch...")
            return True 

    def extract_info(self,line):
        parts = line.split()
        if not parts:
            return None, None
        elif "ii" in parts[0]:
            return parts[1], parts[2]
        else:
            return None, None

    def _parse_dpkg_output(self,output):
        data_dict = {}
        # 将输出按行拆分
        lines = output.splitlines()

        # 解析每一行的信息
        for line in lines:
            # 以空格为分隔符拆分每一行        
            name, version = self.extract_info(line.strip())
            if name and version:
                if ":" in name:
                    name = name.split(":")[0]
                if ":" in version:
                    version = version.split(":")[-1]
                data_dict[name] = version

        return data_dict

    def _get_installed_from_cache(self,cache):
        data_dict = {}
        if cache.get_changes():
            cache.clear()
        for pkg in cache:
            if pkg.is_installed:
                version = getattr(pkg.installed, "version", '')
                data_dict[pkg.name] = version
        return data_dict

    def _get_installed_from_dpkg(self):
        local_packages = None
        process = subprocess.Popen("dpkg -l", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = process.communicate()
        if not error:
            local_packages = self._parse_dpkg_output(output.decode('utf-8'))
        else:
            logging.warning(str(error))

        return local_packages

    def is_customized_system(self,local_data,check_data):
        force_update = check_data.get("force_update_default",False)
        local_version = local_data.values()
        local_packages = local_data.keys()

        def find_key_by_value(my_dict, value):
            return next((key for key, val in my_dict.items() if val == value), None)
        
        def find_kn(string):
            pattern = re.compile(r'k([0-9])')
            match = pattern.search(string)
            
            if match:
                return match.group(0)
            else:
                return None

        # 定制包版本号 标识识别 
        customized_flag = check_data.get("customized_packages_flag",[])
        if customized_flag:
            for ver in local_version:
                # 检查是否包含k[0-9] 的字符串
                result = find_kn(ver)
                if result:
                    ver_split = ver.split(result, 1)[1]
                    for flag in customized_flag:
                        if flag in ver_split:
                            pkg_name = find_key_by_value(local_data,ver)
                            logging.info("Customized version system...")
                            logging.info("Customized: %s:%s includes a custom logo:%s",pkg_name,ver,flag)
                            return True,force_update
                    
        # 定制包名称识别 存在定制包
        customized_list = check_data.get("customized_packages_list",[])
        for pkg in customized_list:
            if pkg in local_packages:
                logging.info("Customized version system...")
                logging.info("Customized Packages(%s)",pkg)
                return True,force_update
        logging.info("Universal version system...")
        return False,force_update


    def is_intermediate_version(self,local_data,check_data,packages_database):
        local_packages = local_data.keys()
        force_update = True

        # 包版本匹配 加入匹配度计算
        packages_version = check_data.get("packages_group",{})
        for key, sub_dict in packages_version.items():
            not_match = []
            mismatch_threshold = sub_dict.setdefault("mismatch_threshold",0)
            if not isinstance(mismatch_threshold, int):
                logging.info("Required match rate not is int type...")
                continue
            
            force_update = sub_dict.get("force_update_popup",True)

            packages_list = sub_dict.get("packages_list",[])

            for pkg in list(set(local_packages) & set(packages_list)):
                database_version = packages_database.get(pkg,[])
                if not database_version:
                    continue
                if local_data[pkg] not in database_version:
                    
                    not_match.append(pkg+":"+local_data[pkg])
                    if mismatch_threshold == 0:
                        logging.warning("Version mismatch num:%d,packages list:%r",len(not_match),not_match)
                        return False,force_update
                    
                    if len(not_match) >= mismatch_threshold:
                        logging.warning("Version mismatch num:%d,packages list:%r",len(not_match),not_match)
                        return False,force_update

        return False,force_update
    
class IncompThirdPkgsCheck:
    def __init__(self):
        self.incomp_json = "/usr/share/kylin-update-desktop-config/version/incomp-third-pkgs.json"

    def check(self,cache,pkgs_list):
        if not os.path.exists(self.incomp_json):
            logging.warning("Skip: pre-upgrade application and patch compatibility detection...")
            return False

        try:
            with open(self.incomp_json,'r') as f:
                thirds_database = json.load(f)
        except Exception as exc:
            logging.warning(str(exc))
        
        thirds_pkgs = list(thirds_database.keys())

        for pkg in thirds_pkgs:
            if pkg in pkgs_list:
                pkg_obj = cache[pkg]
                candidate = getattr(pkg_obj.candidate, "version","")
                version_list = thirds_database[pkg]
                if candidate in version_list:
                    logging.error("Third pkgs(%s:%s) in incompatible conf...",pkg,candidate)
                    raise UpdateBaseError(ERROR_INCOMP_THIRDS_PKGS)


if __name__ == "__main__":
    test = IncompThirdPkgsCheck()
    test.incomp_json = "/home/x/share/kylin-system-updater/backend/tests/test-json/incomp-third-pkgs.json"
    test.check()